import { connect } from 'amqp-connection-manager';

function compact(source) {
    return source.filter((s) => !!s);
}
function combine(opts = '', ...params) {
    let options = { joinWith: ' ' };
    if (typeof opts === 'object') {
        options = opts;
    }
    else {
        params = [opts, ...params];
    }
    const { joinWith } = options;
    return compact(params).join(joinWith);
}

class Connection {
    #urls = [];
    #amqpConnectionManager;
    constructor({ urls, ...options }) {
        if (!Array.isArray(urls)) {
            urls = [urls];
        }
        this.#urls = urls.map(this.#convertUrl);
        this.#amqpConnectionManager = connect(this.#urls, options);
    }
    get connection() {
        return this.#amqpConnectionManager;
    }
    get urls() {
        return this.#urls;
    }
    createChannel({ json = true, ...rest } = {}) {
        return this.#amqpConnectionManager.createChannel({
            json,
            ...rest,
        });
    }
    close() {
        return this.#amqpConnectionManager.close();
    }
    #convertUrl(url) {
        if (typeof url === 'string') {
            return url;
        }
        const { host, port = '', username, password, virtualHost } = url;
        return combine({ joinWith: '/' }, combine({ joinWith: '@' }, combine({ joinWith: '' }, 'amqp://', combine({ joinWith: ':' }, username, password)), combine({ joinWith: ':' }, host, port.toString())), virtualHost);
    }
}

export { Connection };
